import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/enums/layout-mode.enum.dart';
import 'package:panda/models/form-result.model.dart';
import 'package:rxdart/rxdart.dart';

class PandaController {
  final PandaLayoutModesEnum? layoutMode;
  final BehaviorSubject<PandaFormResultModel?> _onChange = BehaviorSubject<PandaFormResultModel?>();
  final BehaviorSubject<bool> _onValidChange = BehaviorSubject<bool>();
  final BehaviorSubject<int> _onNavigateToSection = BehaviorSubject<int>();

  PandaFormResultModel? formResultValue;
  late bool valid;
  PandaCubit? _bloc;

  Stream<PandaFormResultModel?> get onChange => _onChange.stream;

  Stream<bool> get onValidChange => _onValidChange.stream;

  Stream<int> get onNavigateToSection => _onNavigateToSection.stream;

  PandaController({
    this.layoutMode,
  }) {
    onChange.listen((formResult) {
      formResultValue = formResult;
    });
    onValidChange.listen((valid) {
      this.valid = valid;
    });
  }

  void dispose() {
    if (!_onChange.isClosed) {
      _onChange.close();
    }
    if (!_onValidChange.isClosed) {
      _onValidChange.close();
    }
    if (!_onNavigateToSection.isClosed) {
      _onNavigateToSection.close();
    }
  }

  set bloc(PandaCubit? bloc) => _bloc = bloc;

  void change(PandaFormResultModel? formResult) {
    _onChange.add(formResult);
  }

  void validChange(bool valid) {
    if (!_onValidChange.isClosed) _onValidChange.add(valid);
  }

  void navigateToSection(int index) {
    _onNavigateToSection.add(index);
  }

  void submit() => _bloc!.submit();
}
