import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:panda/controllers/panda.controller.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/types/view-config-tile-builder.dart';
import 'package:panda/utils/transition.utils.dart';
import 'package:panda/widgets/section-navigator.dart';
import 'package:panda/widgets/top-level-groups-state.widget.dart';

// Multi Section Layout expects all top level view configs to be of type group.
class PandaMultiSectionLayout extends StatefulWidget {
  final PandaController controller;
  final ViewConfigTileBuilder? viewConfigTileBuilder;

  const PandaMultiSectionLayout({Key? key, required this.controller, this.viewConfigTileBuilder}) : super(key: key);

  @override
  _PandaMultiSectionLayoutState createState() => _PandaMultiSectionLayoutState();
}

class _PandaMultiSectionLayoutState extends State<PandaMultiSectionLayout> {
  @override
  void initState() {
    super.initState();
    widget.controller.onNavigateToSection.listen((int sectionIndex) async {
      _openSectionNavigator(sectionIndex);
    });
  }

  @override
  Widget build(BuildContext context) {
    return TopLevelGroupsState(
      viewConfigTileBuilder: widget.viewConfigTileBuilder,
      onTileTap: (index) => _openSectionNavigator(index),
    );
  }

  void _openSectionNavigator(int index) {
    final data = MediaQuery.of(context);
    Navigator.of(context).push(
      PageRouteBuilder<PandaSectionNavigator>(
        opaque: false,
        transitionDuration: const Duration(milliseconds: 500),
        transitionsBuilder: PandaTransitionUtils.slideUpTransition,
        pageBuilder: (_, __, ___) => BlocProvider.value(
          value: PandaCubit.of(context),
          child: MediaQuery(
            data: data,
            child: PandaSectionNavigator(index),
          ),
        ),
      ),
    );
  }
}
