import 'dart:collection';

import 'package:panda/utils/date-time.utils.dart';

class PandaFormResultModel {
  String? id;
  String? formDescriptorId;
  LinkedHashMap<String, dynamic>? value = LinkedHashMap<String, dynamic>();

  DateTime? createdAt;
  DateTime? updatedAt;

  PandaFormResultModel({
    this.id,
    this.formDescriptorId,
    this.value,
    this.createdAt,
    this.updatedAt,
  });

  PandaFormResultModel shallowClone() {
    return PandaFormResultModel(
      id: id,
      formDescriptorId: formDescriptorId,
      value: value,
      createdAt: createdAt,
      updatedAt: updatedAt,
    );
  }

  PandaFormResultModel.fromJson(LinkedHashMap<String, dynamic>? json) {
    if (json == null) {
      return;
    }

    id = json["_id"];
    formDescriptorId = json["formDescriptorId"];
    value = json["value"];
    createdAt = PandaDateTimeUtils.nullAwareTryParse(json["createdAt"])?.toLocal();
    updatedAt = PandaDateTimeUtils.nullAwareTryParse(json["updatedAt"])?.toLocal();
  }

  static List<PandaFormResultModel> fromJsonList(List<dynamic>? json) {
    return (json ?? []).map((x) => PandaFormResultModel.fromJson(x)).toList();
  }

  Map<String, dynamic> toJson() => {
        if (id != null) "_id": id,
        "formDescriptorId": formDescriptorId,
        "value": value,
      };

  bool? get hasStartedFilling {
    return value?.keys.any((key) => isSectionFilled(key)!);
  }

  int get currentEditingSectionIndex {
    final int? index = value?.keys.toList().lastIndexWhere((key) => isSectionFilled(key)!);
    if (index == null) return 0;
    return index >= 0 ? index : 0;
  }

  bool? isSectionFilled(String keyName) {
    LinkedHashMap<String, dynamic>? section = _getAtKey(keyName);
    return section?.keys.any((key) {
      LinkedHashMap<String, dynamic>? element = section[key];
      if (!(element?.containsKey("value") ?? false)) {
        return false;
      }
      final value = element!["value"];
      if (value == null) {
        return false;
      }
      if (value is List) {
        return value.isNotEmpty;
      }
      return true;
    });
  }

  dynamic _getAtKey(String key) {
    final subKeys = key.split(".");
    dynamic currentValue = value;
    while (subKeys.isNotEmpty) {
      if (currentValue == null) {
        return null;
      }
      final currentKey = subKeys.removeAt(0);
      currentValue = currentValue[currentKey];
    }
    return currentValue;
  }

  dynamic getValueAtKey(String key) {
    final currentValue = _getAtKey(key);
    if (currentValue == null) {
      return null;
    }
    return currentValue["value"];
  }

  void setValueAtKey(String key, dynamic value) {
    final currentValue = _getAtKey(key);
    if (currentValue == null) {
      return;
    }
    currentValue["value"] = value;
  }
}
