import 'dart:collection';

import 'package:panda/models/validation-config/type-config/contains-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/equals-to-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/max-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/min-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/range-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/required-validator.type-config.dart';
import 'package:panda/models/view-config-types/checkbox-type-config.model.dart';
import 'package:panda/models/view-config-types/divider-type-config.model.dart';
import 'package:panda/models/view-config-types/dropdown-type-config.model.dart';
import 'package:panda/models/view-config-types/group-type-config.model.dart';
import 'package:panda/models/view-config-types/short-text-type-config.model.dart';
import 'package:panda/models/view-config-types/table-type-config.model.dart';
import 'package:panda/models/view-config-types/text-type-config.model.dart';

class PandaJsonDeserializers {
  static final Map<Type, dynamic Function(LinkedHashMap<String, dynamic>)> _deserializers = {
    // ViewConfig types
    PandaCheckboxTypeConfigModel: PandaCheckboxTypeConfigModel.fromJson,
    PandaDropdownTypeConfigModel: PandaDropdownTypeConfigModel.fromJson,
    PandaGroupTypeConfigModel: PandaGroupTypeConfigModel.fromJson,
    PandaTableTypeConfigModel: PandaTableTypeConfigModel.fromJson,
    PandaTextTypeConfigModel: PandaTextTypeConfigModel.fromJson,
    PandaShortTextTypeConfigModel: PandaShortTextTypeConfigModel.fromJson,
    PandaDividerTypeConfig: PandaDividerTypeConfig.fromJson,

    // Panda Validator types
    PandaMaxLengthValidatorTypeConfig: PandaMaxLengthValidatorTypeConfig.fromJson,
    PandaMinLengthValidatorTypeConfig: PandaMinLengthValidatorTypeConfig.fromJson,
    PandaRequiredValidatorTypeConfig: PandaRequiredValidatorTypeConfig.fromJson,
    PandaRangeValidatorTypeConfig: PandaRangeValidatorTypeConfig.fromJson,
    PandaEqualsToValidatorTypeConfig: PandaEqualsToValidatorTypeConfig.fromJson,
    PandaContainsValidatorTypeConfig: PandaContainsValidatorTypeConfig.fromJson,
  };

  static T deserialize<T>(LinkedHashMap<String, dynamic> json) {
    if (!_deserializers.containsKey(T)) {
      throw "Unimplemented deserializer function for type ${T.toString()}";
    }

    return _deserializers[T]!(json);
  }

  static bool canDeserialize<T>() {
    return _deserializers.containsKey(T);
  }
}
