import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_exif_rotation/flutter_exif_rotation.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';

class PandaPictureUtils {
  static Future<File?> takePicture(BuildContext context) async {
    return showModalBottomSheet(
      context: context,
      clipBehavior: Clip.hardEdge,
      builder: (context) {
        return SafeArea(
          child: Wrap(
            children: <Widget>[
              ListTile(
                leading: const Icon(Icons.camera_alt),
                title: Text(NiceLocalizations.of(context).translate("panda.camera")),
                onTap: () => _pickFile(context, ImageSource.camera),
              ),
              ListTile(
                leading: const Icon(Icons.image),
                title: Text(NiceLocalizations.of(context).translate("panda.gallery")),
                onTap: () => _pickFile(context, ImageSource.gallery),
              ),
            ],
          ),
        );
      },
    );
  }

  static Future<void> _pickFile(BuildContext context, ImageSource source) async {
    final picker = ImagePicker();

    final pickedFile = await picker.pickImage(source: source, maxWidth: 1000);
    if (pickedFile == null) {
      return null;
    }

    final rotatedImage = await FlutterExifRotation.rotateImage(path: pickedFile.path);
    Navigator.of(context).pop(rotatedImage);
  }
}
