import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:panda/models/view-config-types/dropdown-type-config.model.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';
import 'package:panda/widgets/reactive-error.widget.dart';
import 'package:panda/widgets/submitted-builder.widget.dart';

class PandaDropdownWidget extends PandaBaseWidget {
  const PandaDropdownWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaDropdownWidgetState();
}

class _PandaDropdownWidgetState extends PandaBaseWidgetState<PandaDropdownWidget, PandaDropdownTypeConfigModel> {
  @override
  Widget build(BuildContext context) {
    if (viewConfig == null) const SizedBox.shrink();

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        if (viewConfig!.title?.isNotEmpty ?? false)
          Padding(
            padding: const EdgeInsets.only(bottom: 12),
            child: Text(viewConfig!.title!),
          ),
        Padding(
          padding: const EdgeInsets.only(top: 4, bottom: 12),
          child: _buildDropdownButton(),
        ),
        PandaSubmittedBuilder(
          child: ReactiveError(errorNode: errorNode),
        ),
      ],
    );
  }

  Widget _buildDropdownButton() {
    return StreamBuilder<LinkedHashMap<String, dynamic>?>(
      stream: node!.stream,
      builder: (context, AsyncSnapshot<LinkedHashMap<String, dynamic>?> snapshot) {
        return DropdownButtonFormField<int>(
          isExpanded: true,
          decoration: InputDecoration(
            border: const OutlineInputBorder(),
            labelText: viewConfig!.placeholder,
          ),
          items: (typeConfig?.options ?? []).map((option) {
            return _buildDropdownMenuItem(option);
          }).toList(),
          value: (snapshot.data ?? const {})["value"],
          onChanged: _selectedDropdownValue,
        );
      },
    );
  }

  DropdownMenuItem<int> _buildDropdownMenuItem(PandaDropdownTypeConfigOptionModel option) {
    return DropdownMenuItem<int>(
      child: Text(option.text),
      value: option.value,
    );
  }

  void _selectedDropdownValue(int? optionValue) {
    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    newNodeValue["value"] = optionValue;
    node!.sink.add(newNodeValue);
  }
}
