import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';
import 'package:panda/widgets/reactive-error.widget.dart';
import 'package:panda/widgets/submitted-builder.widget.dart';

class PandaShortTextWidget extends PandaBaseWidget {
  const PandaShortTextWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaShortTextWidgetState();
}

class _PandaShortTextWidgetState extends PandaBaseWidgetState<PandaShortTextWidget, void> {
  @override
  Widget build(BuildContext context) {
    if (viewConfig == null) return const SizedBox.shrink();

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        if (viewConfig!.title?.isNotEmpty ?? false)
          Padding(
            padding: const EdgeInsets.only(bottom: 12),
            child: Text(viewConfig!.title!),
          ),
        Padding(
          padding: const EdgeInsets.only(top: 4, bottom: 12),
          child: _buildTextInput(),
        ),
        PandaSubmittedBuilder(
          child: ReactiveError(errorNode: errorNode),
        ),
      ],
    );
  }

  Widget _buildTextInput() {
    return TextFormField(
      decoration: InputDecoration(
        border: const OutlineInputBorder(),
        labelText: viewConfig!.placeholder,
      ),
      initialValue: node!.value!["value"],
      onChanged: _setValue,
    );
  }

  void _setValue(String newValue) {
    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    newNodeValue["value"] = newValue;
    node!.sink.add(newNodeValue);
  }
}
