import 'dart:collection';
import 'dart:math' as math;

import 'package:flutter/material.dart';
import 'package:hand_signature/signature.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/panda.dart';
import 'package:panda/widgets/reactive-error.widget.dart';
import 'package:panda/widgets/submitted-builder.widget.dart';
import 'package:rxdart/rxdart.dart';

class PandaSignatureWidget extends PandaBaseWidget {
  const PandaSignatureWidget(super.keyTree, {super.key});

  @override
  State<StatefulWidget> createState() => _PandaSignatureWidgetState();
}

class _PandaSignatureWidgetState extends PandaBaseWidgetState<PandaSignatureWidget, void> {
  final _handSignatureControl = HandSignatureControl();

  final _signatureChanged$ = new PublishSubject<void>();

  @override
  void initState() {
    super.initState();

    _signatureChanged$.debounceTime(const Duration(milliseconds: 100)).listen((_) {
      final svg = _handSignatureControl.toSvg(strokeWidth: 3, maxStrokeWidth: 3);

      final newNodeValue = node!.value ?? LinkedHashMap.from({"value": null});
      newNodeValue["value"] = svg;

      node!.sink.add(newNodeValue);
    });
  }

  @override
  void dispose() {
    _handSignatureControl.dispose();
    _signatureChanged$.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: [
          if (viewConfig?.title != null)
            Padding(
              padding: const EdgeInsets.only(bottom: 8),
              child: Text(viewConfig!.title!),
            ),
          DecoratedBox(
            decoration: BoxDecoration(
              border: Border.all(
                color:
                    Theme.of(context).inputDecorationTheme.border?.borderSide.color ?? Theme.of(context).primaryColor,
                width: Theme.of(context).inputDecorationTheme.border?.borderSide.width ?? 1,
              ),
              borderRadius: const BorderRadius.all(Radius.circular(4)),
            ),
            child: Padding(
              padding: const EdgeInsets.all(2),
              child: ClipRRect(
                borderRadius: const BorderRadius.all(Radius.circular(4)),
                child: SizedBox(
                  height: 200,
                  child: GestureDetector(
                    onVerticalDragUpdate: (_) {
                      // NO-OP: Prevent ScrollView scrolling while drawing
                    },
                    child: HandSignature(
                      control: _handSignatureControl,
                      type: SignatureDrawType.line,
                      width: 3,
                      maxWidth: 3,
                      onPointerUp: () => _signatureChanged$.add(null),
                    ),
                  ),
                ),
              ),
            ),
          ),
          const SizedBox(height: 16),
          Align(
            alignment: Alignment.centerLeft,
            child: TextButton(
              onPressed: _handSignatureControl.clear,
              child: Row(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Text(
                    NiceLocalizations.of(context).translate("panda.reset"),
                  ),
                  const SizedBox(width: 8),
                  Transform.rotate(
                    angle: math.pi / 4,
                    child: const Icon(Icons.add),
                  ),
                ],
              ),
            ),
          ),
          const SizedBox(height: 16),
          PandaSubmittedBuilder(
            child: ReactiveError(errorNode: errorNode),
          ),
        ],
      ),
    );
  }
}
