import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/widgets/base-widget.dart';

class PandaTimeWidget extends PandaBaseWidget {
  const PandaTimeWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaTimeWidgetState();
}

class _PandaTimeWidgetState extends PandaBaseWidgetState<PandaTimeWidget, void> {
  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Text(
          viewConfig?.title ?? NiceLocalizations.of(context).translate("panda.no_title"),
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: TextButton(
            onPressed: _onPickTimePressed,
            child: Row(
              children: <Widget>[
                const Icon(Icons.access_time),
                Padding(
                  padding: const EdgeInsets.only(left: 12.0),
                  child: StreamBuilder<LinkedHashMap<String, dynamic>?>(
                    stream: node!.stream,
                    builder: (context, AsyncSnapshot<LinkedHashMap<String, dynamic>?> snapshot) {
                      if (snapshot.hasData && snapshot.data!["value"] != null) {
                        return Text(snapshot.data!["value"]);
                      }

                      return Text(NiceLocalizations.of(context).translate("panda.tap_to_specify_time"));
                    },
                  ),
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }

  Future<void> _onPickTimePressed() async {
    final TimeOfDay? picked = await showTimePicker(
      context: context,
      initialTime: TimeOfDay.now(),
    );

    if (picked == null) {
      return;
    }

    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    newNodeValue["value"] = "${picked.hour}:${picked.minute}";
    node!.sink.add(newNodeValue);
  }
}
