import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/cubit/panda.state.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/types/view-config-tile-builder.dart';

class TopLevelGroupsState extends StatelessWidget {
  final void Function(int) onTileTap;
  final ViewConfigTileBuilder? viewConfigTileBuilder;

  const TopLevelGroupsState({required this.onTileTap, this.viewConfigTileBuilder, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<PandaCubit, PandaState>(
      builder: (_, state) {
        return Column(
          children: (state.formDescriptor?.viewConfigs ?? [])
              .mapIndexed((index, viewConfig) {
                if (viewConfigTileBuilder == null) {
                  return _buildTopLevelGroupRow(state, index, viewConfig);
                }

                return viewConfigTileBuilder!(
                  context,
                  viewConfig,
                  state.formResult?.isSectionFilled(viewConfig.keyName) ?? false,
                  index,
                );
              })
              .toList()
              .expand((child) sync* {
                yield const Divider();
                yield child;
              })
              .skip(1)
              .toList(),
        );
      },
    );
  }

  Widget _buildTopLevelGroupRow(PandaState state, int index, PandaViewConfigModel<dynamic> viewConfig) {
    return ListTile(
      contentPadding: EdgeInsets.zero,
      leading: CircleAvatar(
        backgroundColor: Colors.lightBlueAccent.withOpacity(0.2),
        child: Text(
          "${index + 1}",
          style: const TextStyle(
            fontSize: 18.0,
            fontWeight: FontWeight.w800,
            color: Colors.blueAccent,
          ),
        ),
      ),
      title: Text(
        viewConfig.title ?? "",
        style: const TextStyle(
          fontWeight: FontWeight.bold,
        ),
      ),
      subtitle: (viewConfig.description != null) ? Text(viewConfig.description!) : null,
      trailing: _buildCompletedIcon(state, viewConfig),
      onTap: () => onTileTap(index),
    );
  }

  Widget _buildCompletedIcon(PandaState state, PandaViewConfigModel<dynamic> viewConfig) {
    return Builder(
      builder: (context) {
        Color? color;
        IconData icon;
        bool showBackground;
        if (state.formResult?.isSectionFilled(viewConfig.keyName) ?? false) {
          color = Theme.of(context).primaryColor;
          icon = Icons.check;
          showBackground = true;
        } else {
          color = Colors.grey;
          icon = Icons.edit;
          showBackground = false;
        }

        return SizedBox(
          width: 40.0,
          height: 40.0,
          child: Stack(
            children: <Widget>[
              if (showBackground)
                Positioned.fill(
                  child: Container(
                    decoration: BoxDecoration(
                      color: color.withOpacity(0.2),
                      borderRadius: BorderRadius.circular(32.0),
                    ),
                  ),
                ),
              Center(
                child: Padding(
                  padding: const EdgeInsets.all(10.0),
                  child: Text(
                    String.fromCharCode(icon.codePoint),
                    style: TextStyle(
                      fontFamily: icon.fontFamily,
                      package: icon.fontPackage,
                      fontSize: 18.0,
                      color: color.withOpacity(0.8),
                    ),
                  ),
                ),
              ),
            ],
          ),
        );
      },
    );
  }
}
